{
 copyright   : daniel lorch 1999 <daniel@lorch.cc> (deutsch, english understood)
               http://raylight.hypermart.net
 name        : raylight Autostart (TRaylightAutostart)
 language    : everything greater than delphi 2.0 (only on 2.0 tested)
 description : allows you to start your applications automatically
               on windows-startup (autostart) with a matter of a single mouse click.
               just change the property 'autostart' (truefalse) and your program
               will start up every time windows starts up. TRaylightAutostart will
               automatically write required settings into the registry (or remove
               them if needed).
 legal       : this component ist credit-ware. credit-ware inherits directly
               from public domain software what means that you are allowed
               to use it without any restrictions. the only difference is
               that you have to give proper credits (in readme or in the program
               itself. suggested way is to use TRaylightUrlLabel (also available
               on my homepage) which allows you to have 'hyperlinks' in you programs
               and link to my homepage. i'd also appreciate if you could send me
               a little note where to get your program so i can have a
               look at it.
               I take no responsibility for any damage or consequences directly or
               indirectly, intentionally or unintentionally, which may arise from
               using this program. YOU USE IT AT YOUR OWN RISK!
 history     : (dd/mm/yyyy) format
               1.00 (08.03.1999) daniel lorch <daniel@lorch.cc>:
                -initial release
               1.01 (08.03.1999) Maria Pascal Bihler <PascalBihler@swol.de>
                -Security: No RunTimeError if No Rights to read/Write Registry
                -Handling: If you change AppName, it will be automatically refreshed;
                -Property: StartNextLogon
                -Property: Global
                -Property: Parameters
               1.02 (08.04.1999) John L. McMinn <jmcminn@bigfoot.com>
                -Handling: Enclosed pathnames in parentheses to avoid truncation


 modifying   : if you've made any improvements, first add your name to the
               'history' (above), make a brief description of all the changes
               you've made, increase version number by 0.01 and add the current date.
               then send me a copy of it (if you want). you can also check my
               homepage for recent improvements.
 usage       : raylight Autostart is an non-windowed (= invisible) component.
               new properties are:
               -autostart (boolean): start program every time windows is started up
               -StartNextLogon (bool): start program next time windows log on (only once)
               -appName (string)   : this must be an unique name identyfying your
                                     application. raylight autostart requires this
                                     string to save settings into the registry. usually
                                     the name of your application is sufficient.
               -global (Boolean)   : Sets whether the configuration is only enabled for the current
                                     user or for all users at the PC (Admin-Rights needed);

               sample code:

                procedure TForm1.FormCreate(Sender: TObject);
                begin
                 checkbox1.caption := 'Start this application on windows startup.';
                 checkbox1.checked := RaylightAutostart1.autostart;
                end;

                procedure TForm1.CheckBox1Click(Sender: TObject);
                begin
                 RaylightAutostart1.autostart := checkbox1.checked;
                end;
}

unit raylightAutostart;

interface

uses
  Windows, Messages, SysUtils, Controls, Forms, Graphics, Dialogs,
  StdCtrls, Classes, registry, shellapi, Clipbrd, Menus;

type
  TRaylightAutostart = class(TComponent)
  private
    FAppName   : string;
    FAutoStart : boolean;  
    FStartNextLogon : boolean;
    FGlobal : boolean;


    function WriteToReg(Once,bGlobal,value:Boolean):Boolean;
    function ReadFromReg(Once,bGlobal:Boolean): boolean;

    function checkAutostart: boolean;
    procedure writeAutostart(value: boolean);
    function checkStartNextLogon: boolean;
    procedure writeStartNextLogon(value: boolean);

    function GetFGlobal:Boolean;
    procedure SetFGlobal(value: Boolean);

    procedure SetFAppName(AName:String);

  protected
    { Protected-Deklarationen }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property appName   : string read FAppName write SetFAppName;
    property autostart : boolean read checkAutostart write writeAutostart;
    property StartNextLogon : boolean read checkStartNextLogon write writeStartNextLogon;
    property Global : boolean read GetFGlobal write SetFGlobal;
  end;

procedure Register;

implementation

{$R raylightAutostart.res}

procedure TRaylightAutostart.SetFAppName(AName:String);
var changeit:Array[0..3]of boolean;
begin
 if uppercase(aName)<>Uppercase(FAppName) then {Do only something if the Name changed}
  begin

  {Test which entrys to change}
  changeit[0]:=readFromreg(false,false);  
  changeit[1]:=readFromreg(false,true);
  changeit[2]:=readFromreg(true,false);
  changeit[3]:=readFromreg(true,true);

  {Delete old values}
  if changeit[0] then writeToReg(false,false,false);
  if changeit[1] then writeToReg(false,true,false);
  if changeit[2] then writeToReg(true,false,false);
  if changeit[3] then writeToReg(true,true,false);

  FAppName:=aName;

  {Write New Values}
  if changeit[0] then writeToReg(false,false,true);
  if changeit[1] then writeToReg(false,true,true);
  if changeit[2] then writeToReg(true,false,true);
  if changeit[3] then writeToReg(true,true,true);

  checkAutostart;
  checkStartNextLogon;
 end;
end;


function TRaylightAutostart.GetFGlobal:Boolean;
begin
  Result:=FGlobal;
end;

procedure TRaylightAutostart.SetFGlobal(value: Boolean);
var reg:Tregistry;
begin
  if (value=true) then  {Check whether Admin right or not}
   begin
    reg:=TRegistry.create;
    reg.rootkey:= HKEY_LOCAL_MACHINE;
    if reg.openkey('\Software\Raylight\test\',true) then
     begin
      reg.closekey;
      reg.DeleteKey('\Software\Raylight\test\');
     end
    else value:=FALSE;
    reg.free;
  end;

  if value<>FGlobal then {only do Something if the Value changed}
   begin

    If value=False then
     begin  {Check wether there are global entrys or not}
      if readFromReg(False,true) then
       begin
        WriteToReg(false,false,true);
        WriteToReg(false,true,false);
      end;
      if readFromReg(true,true) then
       begin
        WriteToReg(true,false,true);
        WriteToReg(true,true,false);
      end;
      FGlobal:=FALSE;
     end
    else
     begin {Check wether there are local entrys or not} 
      if readFromReg(False,False) then
       begin
        WriteToReg(false,true,true);
        WriteToReg(false,false,false);
      end;
      if readFromReg(true,False) then
       begin
        WriteToReg(true,true,true);
        WriteToReg(true,false,false);
      end;
      FGlobal:=TRUE;
    end;

  end;
end;


function TRaylightAutostart.checkAutostart: boolean;
begin
  Result:=ReadFromReg(False,FGlobal);
  FAutoStart:=Result;
end;

function TRaylightAutostart.checkStartNextLogon: boolean;
begin
  Result:=ReadFromReg(True,FGlobal);
  FStartNextLogon:=Result;
end;

function TRaylightAutostart.ReadFromReg(Once,bGlobal:Boolean): boolean;
var i:Integer;
    s:String;
begin
 if FAppName <> '' then
 with TRegistry.Create do
 try
  if bGlobal then RootKey := HKEY_LOCAL_MACHINE
  else RootKey := HKEY_CURRENT_USER;

  if (once and OpenKey('\SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnce', false)) OR
   (not(once) and OpenKey ('\SOFTWARE\Microsoft\Windows\CurrentVersion\Run', false)) then
    begin
     if ReadString(FAppName) = ''
      then Result := false
      else
        Result := true;
      CloseKey;
    end
   else result:=FALSE;
 finally
  Free;
 end;

end;

procedure TRaylightAutostart.writeAutostart(value: boolean);
begin
  FAutoStart:=WriteToReg(False,FGlobal,Value);
end;

procedure TRaylightAutostart.writeStartNextLogon(value: boolean);
begin
  FStartNextLogon:=WriteToReg(True,FGlobal,Value);
end;

function TRaylightAutostart.WriteToReg(Once,bGlobal,value:Boolean):Boolean;
const
 firsttime : boolean = true;
 var
 AppExe:string;
begin
 {
  when delphi creates the component it first calls 'writeAutostart' instead
  of 'readAutostart'. but as we first want to check the registry rather
  than using values that were in the component's constructor and overwrite
  previous settings saved in the registry, i've programmed this workaround.
 }
 if firsttime then
 begin
  value := checkAutostart;
  firsttime := false;
 end;

 if FAppName <> '' then
 with TRegistry.Create do
 try
  if bGlobal then RootKey := HKEY_LOCAL_MACHINE
  else RootKey := HKEY_CURRENT_USER;
  if (once and OpenKey ('\SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnce', true)) or
     (not(once) and OpenKey ('\SOFTWARE\Microsoft\Windows\CurrentVersion\Run', true)) then
   begin
    if value then
  begin
  AppExe:=#34+Application.Exename+#34; // allows spaces in the path, jlm
   WriteString(FAppName, AppExe);
     Result := true;
    end else begin
     DeleteValue(FAppName);
     Result := false;
    end;

    CloseKey;
   end
  else Result:=FALSE;
 finally
  Free;
 end;
end;

constructor TRaylightAutostart.create(AOwner: TComponent);
begin
 inherited Create(AOwner);

if FAppName='' then
  FAppName := Application.Title+'#Autostart';
end;

destructor TRaylightAutostart.destroy;
begin
 inherited destroy;

 //nothing to do
end;

procedure Register;
begin
  RegisterComponents('Raylight Software', [TRaylightAutostart]);
end;

end.
